/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * ComponentConfiguration.hxx
 *
 * Authors: Paolo Badino <paolo.badino@cern.ch>
 * Version info: $Id: ComponentConfiguration.hxx,v 1.5 2008/01/24 08:43:43 ptedesco Exp $
 * Release: $Name: glite-data-config-service_R_2_0_0_3 $
 *
 */
 
#ifndef GLITE_DATA_CONF_SERVICE_COMPONENTCONFIGURATION_HXX_
#define GLITE_DATA_CONF_SERVICE_COMPONENTCONFIGURATION_HXX_

#include <string>
#include <map>

namespace glite {
namespace config{
  
/**
 * ComponentConfiguration
 * This interface has to be implemented by all the GLite components that should 
 * be loaded dynamically using the ServiceConfigurator
 * @stereotype interface
 */  
class ComponentConfiguration {
public:
    /**
     * Base class for the Configuration Parameters
     */
    class Param {
    public:
        /**
         * Returns Param Name
         * @return Param Name
         */
        const char * getName() const {return _name.c_str();}
        
        /**
         * Returns Param Type
         * @return Param Type
         */
        virtual const char * getType() const = 0;
        
        /**
         * Destructor
         */
        virtual ~Param(){}
    protected:
        /**
         * Constructor
         * @param name [IN] the name of the Configuration Parameter
         */
        Param(const char * name):_name(name){}
    private:
        /**
         * Default Constructor
         * Not Implemented
         */
        Param();
    private:
        /**
         * The Configuration Parameter Name
         */
        std::string _name;        
    };
    
    /**
     * ComponentConfiguration Map Type
     */
    typedef std::map<std::string, Param *> Params; 

    /**
     * Get Component Name
     * @return the Component Name
     */
    const char * getName() const {return _name.c_str();}
    
    /**
     * Initialize the Component. Parameters passed during the Initialization 
     * are not supposed to change
     * @param params [IN] the parameter for the component's initialization
     * @return 0 if success, -1 otherwsie
     */
    virtual int init(const Params& params) = 0;

    /**
     * Configure the Component. This method is also use to reconfigure the 
     * component after a stop. Parameters passed during the Configuration
     * could be modified on a reconfiguration
     * @param params [IN] the parameter for the component's initialization
     * @return 0 if success, -1 otherwsie
     */
    virtual int config(const Params& params) = 0;
    
    /**
     * Start the Component. This method is called to start the execution or 
     * restart it after a reconfiguration
     * @return 0 if success, -1 otherwsie
     */
    virtual int start() = 0;

    /**
     * Stop the Component. This method is called to stop the execution either 
     * is case of shoutdown and in case of reconfiguration
     * @return 0 if success, -1 otherwsie
     */
    virtual int stop() = 0;
    
    /**
     * Finalize the Component
     * @return 0 if success, -1 otherwsie
     */
    virtual int fini() = 0;

protected:

    /**
     * Constructor
     * @param name [IN] the Component Name
     */
    ComponentConfiguration(const char * name):_name(name){}
    
    /**
     * Destructor
     */
    virtual ~ComponentConfiguration(){}
      
private:
    
    /*
     * Not Implemented
     */
    ComponentConfiguration();
    
private:

    /**
     * Component Name Property
     */
    std::string _name;    
};

} // End namespace config
} // End namespace glite

extern "C" {
/**
 * Create Component instance
 * @return the instance of the component
 */
glite::config::ComponentConfiguration * create_glite_component();

/**
 * Destroy Component instance
 * @param component [IN] the instance of the component to be destroyed
 */
void destroy_glite_component(glite::config::ComponentConfiguration * component);

}

#endif //GLITE_DATA_CONF_SERVICE_COMPONENTCONFIGURATION_HXX_
